#include <cstdio>
#include <iostream>
#include <cassert>
#include <vector>
#include <map>
#include <string>
#include <algorithm>
#include <sstream>
#include "testlib.h"

using namespace std;

int cur_test = -1;
const int GROUPS_CNT = 4;
const int MAXT[GROUPS_CNT] = {100, 1e5, 1e5, 1e5};
const int MAXN[GROUPS_CNT] = {100, 1e6, 1e6, 1e9};


string itoa(int x) 
{
    stringstream ss;
    string s;
    ss << x;
    ss >> s;
    return s;
}

int atoi(string s) 
{
    stringstream ss;
    int x;
    ss << s;
    ss >> x;
    return x;
}

void write_test(vector<pair<int, int>> test) 
{
    startTest(cur_test++);
    cerr << cur_test << endl;
    cout << test.size() << endl;
    for (int i = 0; i < (int)test.size(); i++)
        cout << test[i].first << ' ' << test[i].second << endl;
}

void gen_rand(int t, int maxn)
{
    map<pair<int, int>, bool> used;
    vector<pair<int, int>> test(t);
    for (int i = 0; i < t; i++)
    {
        test[i].first = rnd.next(1, maxn);
        test[i].second = rnd.next(1, maxn);
        while (used[test[i]])
            test[i].first = test[i].first % maxn + 1;
        used[test[i]] = true;
    }
    write_test(test);
}

void gen_1n(int t, int start = 1)
{
    vector<pair<int, int>> test(t);
    for (int i = 0; i < t; i++)
    {
        test[i].first = 1;
        test[i].second = i + start;
        if (rnd.next(2))
            swap(test[i].first, test[i].second);
    }
    shuffle(test.begin(), test.end());
    write_test(test);
}

void gen_max(int t, int maxn)
{
    vector<pair<int, int>> test;
    for (int i = 0; i < t && (int)test.size() < t; i++)
    {
        pair<int, int> cur;
        cur.first = maxn - i;
        for (int j = 0; j < i && (int)test.size() < t; j++)
        {
            cur.second = maxn - j;
            if (rnd.next(2))
                swap(cur.first, cur.second);
            test.push_back(cur);
        }
    }
    shuffle(test.begin(), test.end());
    write_test(test);
}

void gen_equal(int t, int start = 1)
{
    vector<pair<int, int>> test(t);
    for (int i = 0; i < t; i++)
    {
        test[i].first = i + start;
        test[i].second = i + start;
        if (rnd.next(2))
            swap(test[i].first, test[i].second);
    }
    shuffle(test.begin(), test.end());
    write_test(test);
}


int main(int argc, char * argv[]) 
{
    registerGen(argc, argv, 1);
    cur_test = atoi(argv[1]);
    //1
    gen_rand(MAXT[0], MAXN[0]);
    gen_rand(MAXT[0], MAXN[0]);
    gen_1n(MAXT[0]);
    gen_max(MAXT[0], MAXN[0]);
    gen_equal(MAXT[0]);
    // 2 equal
    gen_equal(MAXT[1]);
    gen_equal(MAXT[1], MAXT[1]);
    gen_equal(MAXT[1], MAXN[1] / 2);
    gen_equal(MAXT[1], MAXN[1] - MAXT[1]);
    //3, 4
    for (int i = 2; i < 4; i++) 
    {
        gen_rand(MAXT[i], MAXN[i]);
        gen_rand(MAXT[i], MAXN[i]);
        gen_1n(MAXT[i], MAXN[i] - MAXT[i]);
        gen_max(MAXT[i], MAXN[i]);
        gen_equal(MAXT[i], MAXN[i] - MAXT[i]);
    }
    return 0;
}